(function() {
var GridHeaderFilterMode = { List: 1, CheckedList: 2, DateRangePicker: 3, DateRangeCalendar: 4 };
var GridFilterHelper = ASPx.CreateClass(null, {
 constructor: function(grid) {
  this.grid = grid;
  this.activeElement = null;
  this.lockFilter = true;
  this.searchFilterTimer = -1;
  this.searchEditorInitialValue = undefined;
  this.savedSearchEditorValue_beforeCallback = undefined;
  this.searchEditorLastClientValue = undefined;
  this.searchEditorValueChangedOnCallbackWaiting = false;
  this.headerFilterPopupRelatedColumn = null;
  this.headerFilterPopupDimensions = { };
  this.headerFilterColumnIndicesToInitialize = [ ];
  this.initialHeaderFilterState = { };
  this.SearchEditorKeyDownHandler = this.OnSearchEditorKeyDown.bind(this);
  this.SearchEditorValueChangedHandler = this.OnSearchEditorValueChanged.bind(this);
  this.SearchEditorUserInputHandler = this.OnSearchEditorUserInput.bind(this);
  ASPx.GetControlCollection().ControlsInitialized.AddHandler(this.OnControlsInitialized, this);
 },
 Reset: function() {
  this.initialHeaderFilterState = { };
 },
 IsFilterLocked: function() { return this.lockFilter; },
 LockFilter: function() { this.lockFilter = true; },
 UnlockFilter: function() { this.lockFilter = false; },
 GetActiveElement: function() { return this.activeElement; },
 ResetActiveElement: function() { this.activeElement = null; },
 IsValidInstance: function() { return this.grid.IsValidInstance(); },
 GetAllowSearchFilterTimer: function() { return this.grid.allowSearchFilterTimer; },
 GetSearchFilterDelay: function() { return this.grid.searchFilterDelay; },
 GetEnableAccessibility: function() { return this.grid.accessibilityCompliant; },
 ResolveColumnIndex: function(id) { return this.grid.getColumnIndex(id); },
 GetSearchEditor: function() { return this.grid.GetSearchEditor(); },
 GetCustomSearchPanelEditor: function() { return this.grid.GetCustomSearchPanelEditor(); },
 GetSearchPanelFilter: function() { return this.grid.searchPanelFilter; },
 ApplySearchPanelFilter: function(value) { this.grid.ApplySearchPanelFilter(value, true); },
 GetContentTable: function() { return this.grid.GetMainTable(); },
 OnInit: function() {
  this.LockFilter();
  window.setTimeout(function() { this.UnlockFilter(); }.bind(this), 0);
  this.EnsureSearchEditor();
  window.setTimeout(function() { this.SaveFilterEditorsState(); }.bind(this), 0);
  window.setTimeout(this.InitializeHeaderFilterPopup.bind(this), 0);
 },
 OnAfterCallback: function() {
  this.LockFilter();
  window.setTimeout(function() { this.UnlockFilter(); }.bind(this), 0);
  this.EnsureSearchEditor();
  window.setTimeout(function() { this.SaveFilterEditorsState(); }.bind(this), 0);
  window.setTimeout(this.InitializeHeaderFilterPopup.bind(this), 0);
 },
 OnBeforeCallback: function() {
  this.LockFilter();
  this.UnloadHeaderFilterPopup();
  var edit = this.GetSearchEditor();
  if(edit)
   this.savedSearchEditorValue_beforeCallback = this.GetEditorStringValue(edit);
 },
 OnCancelCallbackCore: function() {
  this.UnlockFilter();
  this.InitializeHeaderFilterPopup();
 },
 SaveFilterEditorsState: function() { },
 ClearFilterEditorsState: function() { },
 OnDispose: function() {
  ASPx.GetControlCollection().ControlsInitialized.RemoveHandler(this.OnControlsInitialized, this);
 },
 LoadSearchEditorValueOnCallback: function() {
  var edit = this.GetSearchEditor();
  if(edit) {
   var value = this.GetEditorStringValue(edit);
   this.searchEditorValueChangedOnCallbackWaiting = value !== this.savedSearchEditorValue_beforeCallback;
   this.searchEditorLastClientValue = value;
   this.savedSearchEditorValue_beforeCallback = undefined;
  }
 },
 SetSearchPanelCustomEditor: function(editor) {
  if(!ASPx.Ident.IsASPxClientEdit(editor) || !editor.GetMainElement())
   return;
  if(this.grid.customSearchPanelEditorID === editor.name)
   return;
  this.RemoveSearchEditorEvents(this.GetSearchEditor());
  this.grid.customSearchPanelEditorID = editor.name;
  this.EnsureSearchEditor();
 },
 AssignSearchEditorEvents: function(edit) {
  if(edit.dxgvFilterHelper === this)
   return;
  if(edit.KeyDown)
   edit.KeyDown.AddHandler(this.SearchEditorKeyDownHandler);
  if(edit.ValueChanged)
   edit.ValueChanged.AddHandler(this.SearchEditorValueChangedHandler);
  if(edit.UserInput)
   edit.UserInput.AddHandler(this.SearchEditorUserInputHandler);
  edit.dxgvFilterHelper = this;
 },
 RemoveSearchEditorEvents: function(edit) {
  if(!edit || edit.dxgvFilterHelper !== this)
   return;
  if(edit.KeyDown)
   edit.KeyDown.RemoveHandler(this.SearchEditorKeyDownHandler);
  if(edit.ValueChanged)
   edit.ValueChanged.RemoveHandler(this.SearchEditorValueChangedHandler);
  if(edit.UserInput)
   edit.UserInput.RemoveHandler(this.SearchEditorUserInputHandler);
  delete edit.dxgvFilterHelper;
 },
 EnsureSearchEditor: function() {
  var edit = this.GetSearchEditor();
  if(!edit) return;
  this.AssignSearchEditorEvents(edit);
  var isCustomEditor = edit === this.GetCustomSearchPanelEditor();
  this.searchEditorInitialValue = isCustomEditor ? this.GetSearchPanelFilter() : this.GetEditorStringValue(edit);
  var applySearchValueOnCallback = this.searchEditorValueChangedOnCallbackWaiting && this.searchEditorInitialValue !== this.searchEditorLastClientValue;
  if(applySearchValueOnCallback) {
   window.setTimeout(function() {
    edit.SetValue(this.searchEditorLastClientValue);
    this.ApplySearchFilterFromEditor(edit);
   }.bind(this), 0);
  }  else {
   if(isCustomEditor)
    window.setTimeout(function() { edit.SetValue(this.GetSearchPanelFilter()); }.bind(this), 0);
  }
  this.searchEditorValueChangedOnCallbackWaiting = false;
 },
 GetEditorStringValue: function(edit) {
  var value = edit.GetValueString();
  return ASPx.IsExists(value) ? value : "";
 },
 OnSearchEditorKeyDown: function(s, e) {
  if(!e.htmlEvent || !this.IsValidInstance())
   return;
  e = e.htmlEvent;
  var clearEditor = e.keyCode == ASPx.Key.Delete && e.ctrlKey;
  if(e.keyCode == ASPx.Key.Enter || clearEditor) {
   if(clearEditor)
    s.SetValue(null);
   this.ApplySearchFilterFromEditor(s);
   ASPx.Evt.PreventEventAndBubble(e);
   return;
  }
  this.CreateSearchFilterTimer(s);
 },
 OnSearchEditorUserInput: function(s, e) {
  if(!this.IsValidInstance() || this.IsSearchFilterTimerActive())
   return;
  this.OnSearchEditorOnPaste(s);
 },
 OnSearchEditorOnPaste: function(editor) {
  this.EnsureEditorRawValue(editor);
  this.CreateSearchFilterTimer(editor);
 },
 OnSearchEditorValueChanged: function(s, e) {
  if(!this.IsValidInstance())
   return;
  window.setTimeout(function() { this.ApplySearchFilterFromEditor(s);  }.bind(this), 0);
 },
 IsSearchFilterTimerActive: function() {
  return this.searchFilterTimer !== -1;
 },
 CreateSearchFilterTimer: function(editor) {
  if(!this.GetAllowSearchFilterTimer()) return;
  this.ClearSearchFilterTimer();
  this.searchFilterTimer = window.setTimeout(function() { this.ApplySearchFilterFromEditor(editor);  }.bind(this), this.GetSearchFilterDelay());
 },
 ClearSearchFilterTimer: function() {
  this.searchFilterTimer = ASPx.Timer.ClearTimer(this.searchFilterTimer);
 },
 ApplySearchFilterFromEditor: function(edit) {
  this.ClearSearchFilterTimer();
  if(this.IsFilterLocked()) return;
  if(!edit || !this.GetContentTable())
   return;
  edit.Validate();
  if(!edit.GetIsValid()) 
   return;
  var value = this.GetEditorStringValue(edit);
  if(value === this.searchEditorInitialValue)
   return;
  this.SaveFilterEditorActiveElement(edit);
  this.ApplySearchPanelFilter(value);
 },
 EnsureEditorRawValue: function(edit) {
  if(ASPx.GetFocusedEditor() === edit && edit.SyncRawValueIfHasTextDecorators) {
   edit.SyncRawValueIfHasTextDecorators();
  }
 },
 SaveFilterEditorActiveElement: function(editor) {
  if(!editor) return;
  var columnIndex = this.ResolveColumnIndex(editor.name);
  if(columnIndex < 0 && editor !== this.GetSearchEditor())
   return;
  this.activeElement = this.GetFilterEditorInputElement(editor);
 },
 GetFilterEditorInputElement: function(editor) {
  if(document.activeElement && !ASPx.Browser.VirtualKeyboardSupported) 
   return document.activeElement;
  if(editor.GetInputElement) 
   return editor.GetInputElement();
  return null;
 },
 GetHeaderFilterPopup: function() { return this.grid.GetHeaderFilterPopup(); },
 GetEnableCallbackAnimation: function() { return this.grid.enableCallbackAnimation; },
 GetEnableHeaderFilterCaching: function() { return this.grid.enableHeaderFilterCaching; },
 GetHeaderFilterIconClassName: function() { return this.grid.HeaderFilterButtonClassName; }, 
 GetColumnHeaders: function(columnIndex) { return [ this.grid.GetHeader(columnIndex, false), this.grid.GetHeader(columnIndex, true) ]; },
 GetColumn: function(columnIndex) { return this.grid._getColumn(columnIndex); },
 GetColumnCount: function() { return this.grid.GetColumnsCount(); },
 ApplyHeaderFilter: function() { this.grid.ApplyHeaderFilterByColumn(); },
 GetCustomizationWindow: function() { return this.grid.GetCustomizationWindow(); },
 ShowLoadingPanelInsideHeaderFilterPopup: function() { this.grid.ShowLoadingPanelInsideHeaderFilterPopup(); },
 GetListBox: function(columnIndex) { 
  var listBox = ASPx.GetControlCollection().Get(this.grid.name + "_HFListBox" + columnIndex); 
  return listBox && !listBox.IsDOMDisposed() ? listBox : null;
 },
 GetSelectAllCheckBox: function(columnIndex) { 
  var checkBox = ASPx.GetControlCollection().Get(this.grid.name + "_HFSACheckBox" + columnIndex); 
  return checkBox && !checkBox.IsDOMDisposed() ? checkBox : null;
 },
 GetDateEdit_From: function(columnIndex) { 
  var dateEdit = ASPx.GetControlCollection().Get(this.grid.name + "_HFFDE" + columnIndex); 
  return dateEdit && !dateEdit.IsDOMDisposed() ? dateEdit : null;
 },
 GetDateEdit_To: function(columnIndex) { 
  var dateEdit = ASPx.GetControlCollection().Get(this.grid.name + "_HFTDE" + columnIndex); 
  return dateEdit && !dateEdit.IsDOMDisposed() ? dateEdit : null;
 },
 GetCalendar: function(columnIndex){ 
  var calendar = ASPx.GetControlCollection().Get(this.grid.name + "_HFC" + columnIndex); 
  return calendar && !calendar.IsDOMDisposed() ? calendar: null;
 },
 GetHeaderFilterPopupRelatedColumn: function() { return this.headerFilterPopupRelatedColumn; },
 GetHeaderFilterButton: function(isCancelButton) {
  var popup = this.GetHeaderFilterPopup();
  if(popup)
   return ASPx.GetControlCollection().Get(isCancelButton ? popup.cpCancelButtonID : popup.cpOkButtonID);
  return null;
 },
 SendHeaderFilterCallback: function(columnIndex, shiftKey) {
  this.grid.gridFuncCallBack([ ASPxClientGridViewCallbackCommand.FilterPopup, this.grid.name, columnIndex, shiftKey ? "T" : ""], this.OnHeaderFilterCallback);
 },
 OnHeaderFilterCallback: function(values) {
  var grid = ASPx.GetControlCollection().Get(values[0]);
  if(grid)
   grid.GetFilterHelper().ProcessHeaderFilterCallback(values[1], values[2]);
 },
 GetHeaderFilterState: function(columnIndex) {
  var listBoxSelectedIndices = [ ];
  var pickerRange = null;
  var calendarDates = null;
  var listBox = this.GetListBox(columnIndex);
  if(listBox)
   listBoxSelectedIndices = listBox.GetSelectedIndices();
  var column = this.GetColumn(columnIndex);
  if(column.HFMode === ASPx.GridHeaderFilterMode.DateRangePicker) {
   var dateEditTo = this.GetDateEdit_To(columnIndex);
   var dateEditFrom = this.GetDateEdit_From(columnIndex);
   if(dateEditTo && dateEditFrom)
    pickerRange = { start: dateEditFrom.GetValue(), end: dateEditTo.GetValue() };
  }
  if(column.HFMode === ASPx.GridHeaderFilterMode.DateRangeCalendar) {
   var calendar = this.GetCalendar(columnIndex);
   if(calendar)
    calendarDates = calendar.GetSelectedDates();
  }
  return { listBoxSelectedIndices: listBoxSelectedIndices, pickerRange: pickerRange, calendarDates: calendarDates };
 },
 IsHeaderFilterChanged: function(columnIndex) {
  var initialState = this.initialHeaderFilterState[columnIndex];
  var currentState = this.GetHeaderFilterState(columnIndex);
  if(!initialState || !currentState)
   return false;
  if(!ASPx.Data.ArrayEqual(initialState.listBoxSelectedIndices, currentState.listBoxSelectedIndices))
   return true;
  var column = this.GetColumn(columnIndex);
  if(column.HFMode === ASPx.GridHeaderFilterMode.DateRangePicker && currentState.pickerRange)
   return initialState.pickerRange.start !== currentState.pickerRange.start || initialState.pickerRange.end !== currentState.pickerRange.end;
  if(column.HFMode === ASPx.GridHeaderFilterMode.DateRangeCalendar && currentState.calendarDates)
   return !ASPx.Data.ArrayEqual(initialState.calendarDates, currentState.calendarDates);
  return false;
 },
 GetHeaderFilterCallbackState: function(columnIndex) {
  var result = [ ];
  var listBox = this.GetListBox(columnIndex);
  if(listBox)
   result = result.concat(listBox.GetSelectedValues());
  var currentState = this.GetHeaderFilterState(columnIndex);
  if(currentState.calendarDates && currentState.calendarDates.length > 0)
   result.push("(Calendar)|" + this.CalcDateRanges(currentState.calendarDates, false).join("|"));
  if(currentState.pickerRange && (currentState.pickerRange.start || currentState.pickerRange.end)) {
   var range = [
    currentState.pickerRange.start && ASPx.DateUtils.GetInvariantDateString(currentState.pickerRange.start) || "",
    currentState.pickerRange.end   && ASPx.DateUtils.GetInvariantDateString(currentState.pickerRange.end)   || ""
   ];
   result.push("(DateRangePicker)|" + range.join("|"));
  }
  return result;
 },
 ProcessHeaderFilterCallback: function(columnIndex, htmlObj) {
  this.headerFilterColumnIndicesToInitialize.push(parseInt(columnIndex));
  this.SetHeaderFilterPopupHtml(htmlObj, columnIndex);
 },
 SetHeaderFilterPopupHtml: function(htmlObj, columnIndex) {
  this.GetHeaderFilterPopup().SetContentHtml(htmlObj, this.GetEnableCallbackAnimation());
 },
 OnControlsInitialized: function() {
  for(var i = 0; i < this.headerFilterColumnIndicesToInitialize.length; i++)
   this.InitHeaderFilter(this.headerFilterColumnIndicesToInitialize[i]);
  this.headerFilterColumnIndicesToInitialize = [ ];
 },
 InitHeaderFilter: function(columnIndex) {
  this.initialHeaderFilterState[columnIndex] = this.GetHeaderFilterState(columnIndex);
  var column = this.GetColumn(columnIndex);
  this.headerFilterPopupRelatedColumn = column;
  this.InitializeListBox(column);
  if(column.HFMode === ASPx.GridHeaderFilterMode.CheckedList)
   this.InitializeSelectAllCheckBox(column);
  if(column.HFMode === ASPx.GridHeaderFilterMode.DateRangePicker)
   this.InitializeDateRangePicker(column);
  if(column.HFMode === ASPx.GridHeaderFilterMode.DateRangeCalendar)
   this.InitializeCalendar(column);
  if(this.GetEnableAccessibility())
   this.InitializeAccessibility(column);
 },
 InitializeListBox: function(column) {
  var listBox = this.GetListBox(column.index);
  if(!listBox) return;
  if(column.HFMode === ASPx.GridHeaderFilterMode.List && this.GetEnableAccessibility())
   listBox.ItemClick.AddHandler(this.OnListBoxSelectionChanged.bind(this));
  else {
   listBox.SelectedIndexChanged.AddHandler(this.OnListBoxSelectionChanged.bind(this));
   listBox.KeyDown.AddHandler(this.ProcessHeaderFilterKeyDown.bind(this));
  }
  var element = listBox.GetListTable ? listBox.GetListTable() : listBox.GetMainElement();
  ASPx.Evt.AttachEventToElement(element, "mousedown", function() { window.setTimeout(ASPx.Selection.Clear, 0); });
  listBox.gvColumn = column;
  if(listBox.cpFSI && listBox.cpFSI.length > 0)
   this.PrepareSeparators(listBox);
  if(this.IsListBoxHasScrolling())
   this.AdjustHeaderFilterListBox(column);
 },
 ProcessHeaderFilterKeyDown: function(listbox, args) {
  this.LockHeaderFilterApply();
  var evt = args.htmlEvent;
  if(evt.keyCode == ASPx.Key.Enter) {
   this.UnlockHeaderFilterApply();
   this.OnListBoxSelectionChanged(listbox);
  }
  window.setTimeout(this.UnlockHeaderFilterApply.bind(this), 100);
 },
 LockHeaderFilterApply: function() {
  this.lockHeaderFilterApply = true;
 },
 UnlockHeaderFilterApply: function() {
  this.lockHeaderFilterApply = false;
 },
 IsHeaderFilterApplyLocked: function() {
  return !!this.lockHeaderFilterApply;
 },
 AdjustHeaderFilterListBox: function(column) {
  var listBox = this.GetListBox(column.index);
  if(listBox && listBox.GetMainElement() && this.RequireShowHeaderFilterListBoxScrollbar(column)) {
   ASPx.SetStyles(listBox.GetMainElement(), { minHeight: this.GetHeaderFilterPopupScrollableElement().offsetHeight });
   listBox.AdjustControl();
  }
 },
 InitializeSelectAllCheckBox: function(column) {
  var checkBox = this.GetSelectAllCheckBox(column.index);
  if(!checkBox) return;
  checkBox.CheckedChanged.AddHandler(function(s) { this.OnSelectAllCheckedChanged(s); }.bind(this));
  checkBox.gvColumn = column;
 },
 InitializeDateRangePicker: function(column) {
  var dateEdit_From = this.GetDateEdit_From(column.index);
  var dateEdit_To = this.GetDateEdit_To(column.index);
  if(!dateEdit_From || !dateEdit_To)
   return;
  dateEdit_From.gvColumn = dateEdit_To.gvColumn = column;
  dateEdit_From.ValueChanged.AddHandler(this.OnDateRangePickerValueChanged.bind(this));
  dateEdit_To.ValueChanged.AddHandler(this.OnDateRangePickerValueChanged.bind(this));
  this.AdjustDateRangePicker(dateEdit_From, dateEdit_To);
 },
 AdjustDateRangePicker: function(dateEdit_From, dateEdit_To) {
  var firstCaptionCell = dateEdit_From.GetCaptionCell();
  var secondCaptionCell = dateEdit_To.GetCaptionCell();
  var width = Math.max(ASPx.GetClearClientWidth(firstCaptionCell), ASPx.GetClearClientWidth(secondCaptionCell));
  ASPx.SetStyles(firstCaptionCell, { minWidth: width });
  ASPx.SetStyles(secondCaptionCell, { minWidth: width });
 },
 InitializeCalendar: function(column) {
  var calendar = this.GetCalendar(column.index);
  calendar.SelectionChanged.AddHandler(this.OnCalendarSelectionChanged.bind(this));
  calendar.gvColumn = column;
 },
 OnListBoxSelectionChanged: function(listBox) {
  if(!listBox) return;
  var column = listBox.gvColumn;
  if(this.RequireApplyHeaderFilterNow(column.HFMode, column.index))
   this.ApplyHeaderFilter();
  else {
   if(column.HFMode === ASPx.GridHeaderFilterMode.CheckedList)
    this.UpdateSelectAllCheckState(column.index);
   this.UpdateOkButtonEnabledState(column.index);
  }
 },
 OnSelectAllCheckedChanged: function(checkBox) {
  var columnIndex = checkBox.gvColumn.index;
  var listBox = this.GetListBox(columnIndex);
  if(checkBox.GetChecked())
   listBox.SelectAll();
  else
   listBox.UnselectAll();
  this.UpdateOkButtonEnabledState(columnIndex);
 },
 OnDateRangePickerValueChanged: function(editor) {
  this.UpdateOkButtonEnabledState(editor.gvColumn.index);
 },
 OnCalendarSelectionChanged: function(editor) {
  this.UpdateOkButtonEnabledState(editor.gvColumn.index);
 },
 RequireApplyHeaderFilterNow: function(mode, columnIndex) {
  return mode === ASPx.GridHeaderFilterMode.List && this.IsHeaderFilterChanged(columnIndex) && !this.IsHeaderFilterApplyLocked();
 },
 UpdateSelectAllCheckState: function(columnIndex) {
  var checkBox = this.GetSelectAllCheckBox(columnIndex);
  var listBox = this.GetListBox(columnIndex);
  if(!checkBox || !listBox) 
   return;
  var selectedItemCount = listBox.GetSelectedIndices().length;
  var checkState = ASPx.CheckBoxCheckState.Indeterminate;
  if(selectedItemCount === 0)
   checkState = ASPx.CheckBoxCheckState.Unchecked;
  else if(selectedItemCount === listBox.GetItemCount())
   checkState = ASPx.CheckBoxCheckState.Checked;
  checkBox.SetCheckState(checkState);
 },
 UpdateOkButtonEnabledState: function(columnIndex) {
  if(this.GetColumn(columnIndex).HFMode !== ASPx.GridHeaderFilterMode.List)
   this.SetHFOkButtonEnabled(this.IsHeaderFilterChanged(columnIndex));
 },
 ApplyHeaderFilterInitialState: function(columnIndex) {
  var state = this.initialHeaderFilterState[columnIndex];
  if(!state) return;
  var column = this.GetColumn(columnIndex);
  var listBox = this.GetListBox(columnIndex);
  if(listBox) {
   if(column.HFMode === ASPx.GridHeaderFilterMode.List) {
    if(this.GetEnableAccessibility() && state.listBoxSelectedIndices.length > 0)
     listBox.SelectIndex(state.listBoxSelectedIndices[0]);
   } else {
    listBox.UnselectAll();
    listBox.SelectIndices(state.listBoxSelectedIndices);
   }
  }
  if(column.HFMode === ASPx.GridHeaderFilterMode.CheckedList)
   this.UpdateSelectAllCheckState(columnIndex);
  if(column.HFMode === ASPx.GridHeaderFilterMode.DateRangePicker) {
   this.GetDateEdit_From(columnIndex).SetValue(state.pickerRange.start);
   this.GetDateEdit_To(columnIndex).SetValue(state.pickerRange.end);
  }
  if(column.HFMode === ASPx.GridHeaderFilterMode.DateRangeCalendar) {
   var calendar = this.GetCalendar(columnIndex);
   calendar.SetValue(null);
   var dates = state.calendarDates || [ ];
   for(var i = 0; i < dates.length; i++)
    calendar.SelectDate(dates[i]);
  }
  this.SetHFOkButtonEnabled(false);
 },
 IsListBoxHasScrolling: function() { return true; }, 
 RequireShowHeaderFilterListBoxScrollbar: function(column) { return this.IsListBoxHasScrolling() && column && (column.HFMode === ASPx.GridHeaderFilterMode.List || column.HFMode === ASPx.GridHeaderFilterMode.CheckedList); },
 GetHeaderFilterPopupScrollableElement: function() { return this.GetHeaderFilterPopup().GetWindowContentElement(-1); },
 InitializeHeaderFilterPopup: function() {
  var popup = this.GetHeaderFilterPopup();
  if(!popup || !popup.GetWindowElement(-1)) 
   return;
  popup.PopUp.AddHandler(this.OnHeaderFilterPopup_PopUp.bind(this));
  popup.CloseUp.AddHandler(this.OnHeaderFilterPopup_CloseUp.bind(this));
  popup.Resize.AddHandler(this.OnHeaderFilterPopup_Resize.bind(this));
  if(this.IsListBoxHasScrolling()) {
   popup.AfterResizing.AddHandler(this.OnHeaderFilter_AfterResizing.bind(this));
   ASPx.Evt.AttachEventToElement(this.GetHeaderFilterPopupScrollableElement(), "scroll", this.OnHeaderFilterPopup_Scroll.bind(this));
  }
  this.InitializeHeaderFilterButtons();
 },
 OnHeaderFilterPopup_PopUp: function(popup) {
  var columnIndex = this.FindColumnIndexByHeaderChild(popup.GetCurrentPopupElement());
  var column = this.GetColumn(columnIndex);
  if(!column) return;
  var shiftKey = popup.GetPopUpReasonMouseEvent().shiftKey;
  var callbackWaiting = this.LoadHeaderFilterPopupData(popup, column, shiftKey);
  if(callbackWaiting) {
   var size = this.GetHeaderFilterPopupSize(columnIndex);
   this.SetHFButtonPanelVisibility(popup, column.HFMode !== ASPx.GridHeaderFilterMode.List);
   if(size) {
    popup.SetSize(size[0], size[1]);
    if(ASPx.Browser.Firefox)
     popup.Shown.AddHandler(function(s) { window.setTimeout(function() { s.SetSize(size[0], size[1]); }, 0); }); 
   }
   this.ShowLoadingPanelInsideHeaderFilterPopup();
  }
  if(this.IsListBoxHasScrolling())
   window.setTimeout(this.UpdateHeaderFilterPopupScroll.bind(this), popup.fadeAnimationDuration);
 },
 LoadHeaderFilterPopupData: function(popup, column, shiftKey) {
  if(column === this.GetHeaderFilterPopupRelatedColumn() && this.UseHeaderFilterContentCaching(column.index) && popup.savedShiftKey === shiftKey) {
   this.ApplyHeaderFilterInitialState(column.index);
  } else {
   this.SendHeaderFilterCallback(column.index, shiftKey);
   popup.SetContentHtml("");
   this.headerFilterPopupRelatedColumn = null;
   popup.savedShiftKey = shiftKey;
   return true;
  }
 },
 UpdateHeaderFilterPopupScroll: function() {
  var column = this.GetHeaderFilterPopupRelatedColumn();
  var disablePopupScroll = this.RequireShowHeaderFilterListBoxScrollbar(column) && this.GetHeaderFilterPopup().IsVisible();
  ASPx.SetStyles(this.GetHeaderFilterPopupScrollableElement(), { overflowY: disablePopupScroll ? "hidden" : "auto" });
 },
 OnHeaderFilterPopup_CloseUp: function(popup) {
  var column = this.GetHeaderFilterPopupRelatedColumn();
  if(column && !this.UseHeaderFilterContentCaching(column.index))
   window.setTimeout(function() { popup.SetContentHtml(""); }, 0);
  this.UpdateHeaderFilterPopupScroll();
 },
 OnHeaderFilterPopup_Resize: function(popup) {
  var columnIndex = this.FindColumnIndexByHeaderChild(popup.GetCurrentPopupElement());
  if(this.GetColumn(columnIndex))
   this.SetHeaderFilterPopupSize(columnIndex, popup.GetWidth(), popup.GetHeight());
 },
 OnHeaderFilter_AfterResizing: function(popup) {
  var column = this.GetHeaderFilterPopupRelatedColumn();
  if(column)
   this.AdjustHeaderFilterListBox(column);
 },
 OnHeaderFilterPopup_Scroll: function(e) {
  var column = this.GetHeaderFilterPopupRelatedColumn();
  if(this.RequireShowHeaderFilterListBoxScrollbar(column))
   this.GetHeaderFilterPopupScrollableElement().scrollTop = 0;
 },
 InitializeHeaderFilterButtons: function() {
  var popup = this.GetHeaderFilterPopup();
  if(!popup) return;
  var buttons = this.GetHeaderFilterButtons();
  for(var i = 0; i < buttons.length; i++) {
   popup.AddPopupElement(buttons[i]);
   if(this.GetEnableAccessibility())
    ASPx.Evt.AttachEventToElement(buttons[i], "keydown", GridFilterHelper.HeaderFilterButtonKeyDownHandler);
  }
 },
 UnloadHeaderFilterPopup: function() {
  var popup = this.GetHeaderFilterPopup();
  if(!popup) return;
  popup.RemoveAllPopupElements();
  popup.Hide();
  if(this.GetEnableAccessibility()) {
   var buttons = this.GetHeaderFilterButtons();
   for(var i = 0; i < buttons.length; i++)
    ASPx.Evt.DetachEventFromElement(buttons[i], "keydown", GridFilterHelper.HeaderFilterButtonKeyDownHandler);
  }
 },
 GetHeaderFilterButtons: function() {
  var buttons = [ ];
  var visibleIndices = this.GetVisibleColumnIndices();
  for(var i = 0; i < visibleIndices.length; i++) {
   var index = visibleIndices[i];
   var headers = this.GetColumnHeaders(index);
   for(var j = 0; j < headers.length; j++)
    this.PopulateHeaderFilterButtons(headers[j], buttons);
  }
  var custWindow = this.GetCustomizationWindow();
  if(custWindow)
   this.PopulateHeaderFilterButtons(custWindow.GetWindowClientTable(-1), buttons);
  return buttons;
 },
 GetVisibleColumnIndices: function() {
  var indices = [ ];
  for(var i = 0; i < this.GetColumnCount(); i++) {
   if(this.GetColumn(i).visible)
    indices.push(this.GetColumn(i).index);
  }
  return indices;
 },
 PopulateHeaderFilterButtons: function(container, buttons) {
  if(!container) return;
  var images = container.getElementsByTagName("IMG");
  for(var i = 0; i < images.length; i++) {
   var button = ASPx.getSpriteMainElement(images[i]);
   if(ASPx.ElementContainsCssClass(button, this.GetHeaderFilterIconClassName()))
    buttons.push(button);
  }
 },
 SetHFButtonPanelVisibility: function(popup, visible) {
  var buttonPanel = document.getElementById(popup.cpButtonPanelID);
  if(buttonPanel) {
   ASPx.SetElementDisplay(buttonPanel, visible);
   this.SetHFOkButtonEnabled(false);
  }
 },
 SetHFOkButtonEnabled: function(enabled) {
  var button = this.GetHeaderFilterButton(false);
  if(button)
   button.SetEnabled(enabled);
 },
 GetHeaderFilterPopupSize: function(key) {
  var size = this.headerFilterPopupDimensions[key];
  if(size) return size;
  if(!this.headerFilterPopupDimensions["Default"]) {
   var popup = this.GetHeaderFilterPopup();
   this.SetHeaderFilterPopupSize("Default", popup.GetWidth(), popup.GetHeight());
  }
  return this.headerFilterPopupDimensions["Default"];
 },
 SetHeaderFilterPopupSize: function(key, width, height) {
  this.headerFilterPopupDimensions[key] = [ width, height ];
 },
 FindColumnIndexByHeaderChild: function(element) {
  if(!element) 
   return -1;
  var level = 0;
  while(level < 6) {
   var index = this.ResolveColumnIndex(element.id);
   if(index > -1)
    return index;
   element = element.parentNode;
   level++;
  }
  return -1;
 },
 UseHeaderFilterContentCaching: function(columnIndex) {
  if(!this.GetEnableHeaderFilterCaching())
   return false;
  var listBox = this.GetListBox(columnIndex);
  return listBox && listBox.GetMainElement() && listBox.GetItemCount() < 1000; 
 },
 CalcDateRanges: function(dates) { 
  var result = [ ];
  dates.sort(function(a, b) { return a - b; });
  var range = { start: null, end: null };
  for(var i = 0; i < dates.length; i++) {
   var date = dates[i];
   if(!range.start || !range.end)
    range.start = range.end = date;
   var isLastDate = dates.length === i + 1;
   var nextDate = ASPxClientCalendar.AddDays(date, 1);
   if(!isLastDate && ASPx.DateUtils.AreDatesEqualExact(nextDate, dates[i + 1])) {
    range.end = nextDate;
   } else {
    result.push(ASPx.DateUtils.GetInvariantDateString(range.start));
    result.push(ASPx.DateUtils.GetInvariantDateString(range.end));
    range.start = range.end = null;
   }
  }
  return result;
 },
 InitializeAccessibility: function(column) {
  var listBox = this.GetListBox(column.index);
  if(listBox) {
   listBox.SelectedIndexChanged.AddHandler(function() { this.MakeListBoxSelectedItemVisible(column); }.bind(this));
   this.MakeListBoxSelectedItemVisible(column);
  }
  var popup = this.GetHeaderFilterPopup();
  if(popup && !popup.gvAccessibilityEventsAssigned) {
   popup.Shown.AddHandler(function() { this.MakeListBoxSelectedItemVisible(this.GetHeaderFilterPopupRelatedColumn()); }.bind(this));
   ASPx.Evt.AttachEventToElement(popup.GetCurrentWindowElement(), "keydown", this.ProcessAccessibleHeaderFilterPopupKeyDown.bind(this));
   popup.gvAccessibilityEventsAssigned = true;
  }
 },
 MakeListBoxSelectedItemVisible: function(column) {
  if(!column || column.HFMode !== ASPx.GridHeaderFilterMode.List)
   return;
  var listBox = this.GetListBox(column.index);
  var itemIndex = listBox.GetSelectedIndex();
  var div = this.GetHeaderFilterPopupScrollableElement();
  var item = listBox.GetItemElement(itemIndex);
  if(!div || !item)
   return;
  var itemHeight = item.offsetHeight;
  var divHeight = parseInt(ASPx.GetCurrentStyle(div).height);
  var itemStart = item.offsetTop;
  var itemEnd = itemStart + itemHeight;
  var divStart = div.scrollTop;
  var divEnd = divStart + divHeight;
  var scrollTop = divStart;
  var isItemAbove = itemStart < divStart;
  var isItemBelow = itemEnd > divEnd;
  if(isItemAbove)
   scrollTop = itemStart;
  else if(isItemBelow)
   scrollTop = itemEnd - divHeight;
  if(div.scrollTop !== scrollTop)
   div.scrollTop = scrollTop;
 },
 ProcessAccessibleHeaderFilterPopupKeyDown: function(e) {
  var keyCode = ASPx.Evt.GetKeyCode(e);
  if(keyCode !== ASPx.Key.Enter) 
   return;
  var handled = true;
  var column = this.GetHeaderFilterPopupRelatedColumn();
  if(column.HFMode === ASPx.GridHeaderFilterMode.List) {
   this.OnListBoxSelectionChanged(this.GetListBox(column.index));
  } else {
   var applyFilterOnEnter = this.IsHeaderFilterChanged(column.index) && !this.IsCancelButtonEventSource(e) && !this.IsDateEditorDroppedDown(column);
   if(applyFilterOnEnter)
    this.ApplyHeaderFilter();
   else
    handled = false;
  }
  if(handled)
   ASPx.Evt.PreventEventAndBubble(e);
 },
 IsCancelButtonEventSource: function(evt) {
  return ASPx.GetIsParent(this.GetHeaderFilterButton(true).GetMainElement(), ASPx.Evt.GetEventSource(evt));
 },
 IsDateEditorDroppedDown: function(column) {
  if(column.HFMode === ASPx.GridHeaderFilterMode.DateRangePicker)
   return this.GetDateEdit_From(column.index).droppedDown || this.GetDateEdit_To(column.index).droppedDown;
  return false;
 },
 PrepareSeparators: function(listBox) {
  if(typeof(ASPxClientListBox) != "undefined" && listBox instanceof ASPxClientListBox)
   GridHFListBoxWrapper.Initialize(listBox);
  for(var i = 0; i < listBox.cpFSI.length; i++) {
   var separatorIndex = listBox.cpFSI[i];
   var item = listBox.GetItemElement(separatorIndex);
   if(ASPx.IsExists(item)) {
    var tr = ASPx.GetParentByTagName(item, "TR");
    this.AppendSeparatorRow(tr);
   }
  }
 },
 AppendSeparatorRow: function(targetRow){
  var newTr = document.createElement("TR");
  ASPx.InsertElementAfter(newTr, targetRow);
  var td = document.createElement("TD");
  var colSpan = this.GetColSpanSum(targetRow);
  if(colSpan > 1)
   td.colSpan = colSpan;
  newTr.appendChild(td);
  var separatorDiv = document.createElement("DIV");
  separatorDiv.className = this.GetFilterValuesSeparatorClassName();
  td.appendChild(separatorDiv);
 },
 GetColSpanSum: function(tableRow){
  var colSpan = 0;
  var cells = ASPx.GetChildNodesByTagName(tableRow, "TD");
  for(var i = 0; i < cells.length; i++)
   colSpan += cells[i].colSpan;
  return colSpan;
 },
 GetFilterValuesSeparatorClassName: function() { return this.grid.GetCssClassNamePrefix() + "HFSD"; },
});
var GridHFListBoxWrapper = {
 Initialize: function(listBox){
  listBox.GetItemRow = GridHFListBoxWrapper.GetItemRow;
  listBox.GetItemCount = GridHFListBoxWrapper.GetItemCount;
  listBox.OnItemClickCore = listBox.OnItemClick;
  listBox.OnItemClick = GridHFListBoxWrapper.OnItemClick;
  listBox.FindInternalCheckBoxIndexCore = listBox.FindInternalCheckBoxIndex;
  listBox.FindInternalCheckBoxIndex = GridHFListBoxWrapper.FindInternalCheckBoxIndex;
 },
 GetItemRow: function(index){
  var itemRows = GridHFListBoxWrapper.GetItemRows(this);
  if(index >= 0)
   return itemRows[index] || null;
  return null;
 },
 OnItemClick: function(index, evt){
  var correctIndex = GridHFListBoxWrapper.GetCorrectItemIndex(this, index);
  this.OnItemClickCore(correctIndex, evt);
 },
 FindInternalCheckBoxIndex: function(element){
  var index = this.FindInternalCheckBoxIndexCore(element);
  return GridHFListBoxWrapper.GetCorrectItemIndex(this, index);
 },
 GetItemCount: function(){
  return GridHFListBoxWrapper.GetItemRows(this).length;
 },
 GetCorrectItemIndex: function(listBox, index){
  for(var i = 0; i < listBox.cpFSI.length; i++) {
   if(listBox.cpFSI[i] < index)
    index--;
  }
  return index;
 },
 GetItemRows: function(listBox){
  var itemRows = [];
  var listTable = listBox.GetListTable();
  var rows = listTable ? listTable.rows : null;
  for(var i = 0; rows && i < rows.length; i++){
   if(ASPx.ElementContainsCssClass(rows[i], "dxeListBoxItemRow"))
    itemRows.push(rows[i]);
  }
  return itemRows;
 }
};
GridFilterHelper.HeaderFilterButtonKeyDownHandler = function(evt) { ASPx.Evt.InvokeMouseClickByKeyDown(evt, ASPx.PopupElementMouseEvent); };
ASPx.GridHeaderFilterMode = GridHeaderFilterMode;
ASPx.GridFilterHelper = GridFilterHelper;
})();
